@@ -7,36 +7,12 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-import re
 from abc import abstractmethod
-from .atomic_types import AtomicTypeABCMeta, AnyAtomicType
+from .atomic_types import AtomicTypeABCMeta
+from .helpers import QNAME_PATTERN
 
 
-class Notation(metaclass=AtomicTypeABCMeta):
-    name = 'NOTATION'
-    pattern = re.compile(
-        r'^(?:(?P<prefix>[^\d\W][\w\-.\u00B7\u0300-\u036F\u0387\u06DD\u06DE\u203F\u2040]*):)?'
-        r'(?P<local>[^\d\W][\w\-.\u00B7\u0300-\u036F\u0387\u06DD\u06DE\u203F\u2040]*)$',
-    )
-
-    @abstractmethod
-    def __new__(cls, *args, **kwargs):
-        raise NotImplementedError()
-
-    @classmethod
-    def validate(cls, value):
-        if isinstance(value, cls):
-            return
-        elif isinstance(value, bytes):
-            value = value.decode()
-        elif not isinstance(value, str):
-            raise cls.invalid_type(value)
-
-        if any(cls.pattern.match(x) for x in value.split()):
-            raise cls.invalid_value(value)
-
-
-class QName(AnyAtomicType):
+class AbstractQName(metaclass=AtomicTypeABCMeta):
     """
     XPath compliant QName, bound with a prefix and a namespace.
 
@@ -44,17 +20,14 @@ class QName(AnyAtomicType):
     URI if a prefixed name is provided for the 2nd argument.
     :param qname: the prefixed name or a local name.
     """
-    name = 'QName'
-    pattern = re.compile(
-        r'^(?:(?P<prefix>[^\d\W][\w\-.\u00B7\u0300-\u036F\u0387\u06DD\u06DE\u203F\u2040]*):)?'
-        r'(?P<local>[^\d\W][\w\-.\u00B7\u0300-\u036F\u0387\u06DD\u06DE\u203F\u2040]*)$',
-    )
+    pattern = QNAME_PATTERN
 
+    @abstractmethod
     def __init__(self, uri, qname):
         if uri is None:
-            self.namespace = ''
+            self.uri = ''
         elif isinstance(uri, str):
-            self.namespace = uri
+            self.uri = uri
         else:
             raise TypeError('the 1st argument has an invalid type %r' % type(uri))
 
@@ -72,21 +45,46 @@ class QName(AnyAtomicType):
             msg = '{!r}: cannot associate a non-empty prefix with no namespace'
             raise ValueError(msg.format(self))
 
+    @property
+    def namespace(self):
+        return self.uri
+
     @property
     def expanded_name(self):
-        if not self.namespace:
-            return self.local_name
-        return '{%s}%s' % (self.namespace, self.local_name)
+        return '{%s}%s' % (self.uri, self.local_name) if self.uri else self.local_name
 
     def __repr__(self):
-        if not self.namespace:
-            return '%s(%r)' % (self.__class__.__name__, self.qname)
-        return '%s(%r, namespace=%r)' % (self.__class__.__name__, self.qname, self.namespace)
+        return '%s(uri=%r, qname=%r)' % (self.__class__.__name__, self.uri, self.qname)
 
     def __str__(self):
         return self.qname
 
+    def __hash__(self):
+        return hash((self.uri, self.local_name))
+
     def __eq__(self, other):
-        if not isinstance(other, self.__class__):
+        if not isinstance(other, AbstractQName):
             raise TypeError("cannot compare {!r} to {!r}".format(type(self), type(other)))
-        return self.namespace == other.namespace and self.local_name == other.local_name
+        return self.uri == other.uri and self.local_name == other.local_name
+
+
+class QName(AbstractQName):
+    """
+    XPath compliant QName, bound with a prefix and a namespace.
+
+    :param uri: the bound namespace URI, must be a not empty \
+    URI if a prefixed name is provided for the 2nd argument.
+    :param qname: the prefixed name or a local name.
+    """
+    name = 'QName'
+
+    def __init__(self, uri, qname):
+        super().__init__(uri, qname)
+
+
+class Notation(AbstractQName):
+    name = 'NOTATION'
+
+    @abstractmethod
+    def __init__(self, uri, qname):
+        super().__init__(uri, qname)
